'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var iconv = _interopDefault(require('iconv-lite'));
var convert = require('@analys/convert');
var matrixTranspose = require('@vect/matrix-transpose');

const CsvReg = (de, qt) => new RegExp(`(\\${de}|\\r?\\n|\\r|^)` + // Delimiters
`(?:${qt}([^${qt}]*(?:""[^${qt}]*)*)${qt}|` + // Quoted fields
`([^"\\${de}\\r\\n]*))` // Standard fields
, 'gi'); // const regexCsv = /(,|\x?\n|\x|^)(?:"([^"]*(?:""[^"]*)*)"|([^",\x\n]*))/gi

// e.g. dqRg = /""/g
const DoubleQuoteReg = qt => new RegExp(qt + qt, 'g');

const parseCsvReg = (tx, de = ',', qt = '"') => {
  const regCsv = CsvReg(de, qt),
        // reg-exp to parse the CSV values, and delimiter default to comma.
  reqDQt = DoubleQuoteReg(qt),
        // reg-exp for double quotes
  rows = []; // data rows as 2-d array

  let matches,
      // array to hold individual pattern matching groups.
  sep,
      // captured separator, can be either delimiter or line-feed.
  qv,
      // captured quoted value.
  nq,
      // captured unquoted value.
  word,
      // final processed capture value.
  row = []; // data row.

  while ((matches = regCsv.exec(tx)) && ( // Loop the reg-exp matches until can find match no more.
  [, sep, qv, nq] = matches)) {
    if (sep && sep !== de) rows.push(row), row = []; // separator is line-feed, push new row.

    row.push(word = qv // check if the captured value is quoted.
    ? qv.replace(reqDQt, qt) // unescape any double quotes.
    : (nq || '').trim() // found a non-quoted value.
    ); // add the value string to the data array.
  }

  return rows; // Return the parsed data rows in 2-d array.
};

const parseCsvMap = (csvText, de = ',', lf = '\n') => {
  return csvText.split(lf).map(row => row.split(de).map(word => word.trim()));
};

const popBlank = rows => {
  let {
    length: l
  } = rows,
      tail;

  while (l) {
    tail = rows[--l];
    if (tail.length && tail.some(x => x === null || x === void 0 ? void 0 : x.trim().length)) break;
    rows.pop();
  }

  return rows;
};

class NaiveCsv {
  /**
   *
   * @param {string|*} csvText
   * @param {string} [de=',']
   * @param {string} [lf='\x\n']
   * @param {string?} [qt]
   * @param {string?} [decode]
   * @param {boolean} [transpose]
   * @param {boolean} [popBlank]
   * @returns {*[][]}
   */
  static toRows(csvText, {
    de = ',',
    lf = '\r\n',
    // lf not applicable if qt is specified
    qt = '\"',
    decode,
    transpose,
    popBlank: popBlank$1
  } = {}) {
    var _ref, _mx, _mx2, _mx3;

    if (decode) csvText = iconv.decode(csvText, decode);
    let mx = qt ? parseCsvReg(csvText, de, qt) : parseCsvMap(csvText, de, lf);
    if (transpose) mx = popBlank$1 ? (_ref = (_mx = mx, popBlank(_mx)), matrixTranspose.transpose(_ref)) : (_mx2 = mx, matrixTranspose.transpose(_mx2));
    return popBlank$1 ? (_mx3 = mx, popBlank(_mx3)) : mx;
  }
  /**
   *
   * @param {string|*} csvText
   * @param {string} [de=',']
   * @param {string} [lf='\x\n']
   * @param {string?} [qt]
   * @param {string?} [decode]
   * @param {boolean} [transpose]
   * @param {boolean} [popBlank]
   * @param {string} [title]
   * @returns {{head, rows, title}}
   */


  static toTable(csvText, {
    de = ',',
    lf = '\r\n',
    qt,
    decode,
    transpose,
    popBlank,
    title
  } = {}) {
    const rows = NaiveCsv.toRows(csvText, {
      de,
      lf,
      qt,
      decode,
      transpose,
      popBlank
    }),
          head = rows.shift();
    return {
      head,
      rows,
      title
    };
  }
  /**
   *
   * @param {string|*} csvText
   * @param {string} [de=',']
   * @param {string} [lf='\x\n']
   * @param {string?} [qt]
   * @param {string?} [decode]
   * @param {boolean} [transpose]
   * @param {boolean} [popBlank]
   * @param {string} [title]
   * @returns {Object[]}
   */


  static toSamples(csvText, {
    de = ',',
    lf = '\r\n',
    qt,
    transpose,
    decode,
    popBlank
  } = {}) {
    const rows = NaiveCsv.toRows(csvText, {
      de,
      lf,
      qt,
      decode,
      transpose,
      popBlank
    }),
          head = rows.shift();
    return convert.tableToSamples({
      head,
      rows
    });
  }

}

exports.NaiveCsv = NaiveCsv;
